import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import java.io.*;
import ij.io.*;
import ij.plugin.*;
import java.util.*;

/** Process all image files in a directory and output mean and SD scores into a text file */

public class NCSU_turfcolor implements PlugIn {
	
	public void run(String arg) {
		String inDir = IJ.getDirectory("Select folder that contains image files ...");
		if (inDir==null)
			return;
		SaveDialog sd = new SaveDialog("Save result ...", "result",".csv");
		String outDir = sd.getDirectory();
		String fileName = sd.getFileName();
		BufferedWriter bw = null;
		try {
			bw = new BufferedWriter(new FileWriter(outDir+fileName));
		} catch (Exception e) {
			IJ.error("ASCII Writer", e.getMessage());
			return;
		}
		if (fileName == null || outDir == null || bw == null)
			return;
		processDir(inDir, bw);
	}

	private void processDir(String inDir, BufferedWriter bw) {
		if (inDir == null || bw == null)
			return;
		String[] list = new File(inDir).list();
		if (list == null)
			return;
		try {
			bw.write("File name, DGCI Mean, DGCI SD, Hue Mean, Hue SD, Saturation Mean, Saturation SD, Brightness Mean, Brightness SD\n");
		} catch (Exception e) {
			IJ.error("ASCII Writer", e.getMessage());
			return;
		}
		for (int i=0; i<list.length; i++) {
			IJ.showProgress(i, list.length);
			IJ.showStatus(i+"/"+list.length);
			boolean isDir = (new File(inDir+list[i])).isDirectory();
			if (!isDir && !list[i].startsWith(".")) {
				ImagePlus img = IJ.openImage(inDir+list[i]);
				if (img==null) continue;
				processFile(img, bw);
			}
		}
		try {
			bw.close();
		} catch (Exception e) {
			IJ.error("ASCII Writer", e.getMessage());
			return;
		}
		IJ.showProgress(1.0);
	}
	
	/** Process individual file and calculate/store mean and SD score
	*/
	private void processFile(ImagePlus imp, BufferedWriter bw) {
		if (imp == null || bw == null)
			return;
		ImageConverter.setDoScaling(true);
		int w = imp.getWidth();
		int h = imp.getHeight();
		byte[] hue,s,b;
		ColorProcessor cp;
		double sum = 0.0d, mean = 0.0d, standardD = 0.0d;
		double hSum = 0.0d, hMean = 0.0d, hSD = 0.0d;
		double sSum = 0.0d, sMean = 0.0d, sSD = 0.0d;
		double bSum = 0.0d, bMean = 0.0d, bSD = 0.0d;

		double[] output = new double[w*h];
		double[] hOutput = new double[w*h];
		double[] sOutput = new double[w*h];
		double[] bOutput = new double[w*h];
		hue = new byte[w*h];
		s = new byte[w*h];
		b = new byte[w*h];
		cp = (ColorProcessor)imp.getProcessor();
		cp.getHSB(hue,s,b);
		
		for (int i=0; i<h; i++) {
			for (int j=0; j<w; j++) {
				int pos = i*w+j;
				// convert (signed) byte to unsigned int first
				double dH = (double)(0xFF & hue[pos])*360/255;
				hOutput[pos] = dH;
				dH = (dH-60)/60;
				double dS = (double)(0xFF & s[pos])/255;
				sOutput[pos] = dS;
				dS = 1 - dS;
				double dB = (double)(0xFF & b[pos])/255;
				bOutput[pos] = dB;
				dB = 1 - dB;
				output[pos] = 0;
				output[pos] += dH;
				output[pos] += dS;
				output[pos] += dB;
				output[pos] /= 3;
				sum += output[pos];
				hSum += hOutput[pos];
				sSum += sOutput[pos];
				bSum += bOutput[pos];
			}
		}
		mean = sum/(w*h);
		hMean = hSum/(w*h);
		sMean = sSum/(w*h);
		bMean = bSum/(w*h);
		standardD = getSD(output, mean);
		hSD = getSD(hOutput, hMean);
		sSD = getSD(sOutput, sMean);
		bSD = getSD(bOutput, bMean);
		try {
			bw.write(imp.getTitle());
			bw.write(", ");
			bw.write(Double.toString(mean));
			bw.write(", ");
			bw.write(Double.toString(standardD));
			bw.write(", ");
			bw.write(Double.toString(hMean));
			bw.write(", ");
			bw.write(Double.toString(hSD));
			bw.write(", ");
			bw.write(Double.toString(sMean));
			bw.write(", ");
			bw.write(Double.toString(sSD));
			bw.write(", ");
			bw.write(Double.toString(bMean));
			bw.write(", ");
			bw.write(Double.toString(bSD));
			bw.write("\n");
		} catch (Exception e) {
			IJ.error("ASCII Writer", e.getMessage());
			return;
		}
	}
	
	/** Get SD based on array of Double type and the mean
	SD is sqrt of sum of (values-mean) squared divided by n
	*/
	private double getSD(double[] data, double mean) 
	{
		if (data == null)
			return Double.NaN;
		final int n = data.length; 
		if (n<2)
			return Double.NaN;
		// calculate the sum of squares 
		double sum = 0; 
		for (int i=0; i<n; i++) { 
			final double v = data[i] - mean; 
			sum += v*v; 
		} 
		return Math.sqrt(sum/n); 
	} 
}